@@ -1,7 +1,7 @@
 #!/usr/bin/perl
 #
 # Minimalist - Minimalistic Mailing List Manager.
-# Copyright (c) 1999-2002 Vladimir Litovka <doka@kiev.sovam.com>
+# Copyright (c) 1999-2005 Vladimir Litovka <vlitovka@gmail.com>
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -29,7 +29,7 @@
 use Fcntl ':flock';	# LOCK_* constants
 use integer;
 
-$version = '2.5(2) (Chernigov White)';
+$version = '2.5(3) (Good Bowl)';
 $config = "/usr/local/etc/minimalist.conf";
 
 # Program name and arguments for launching if commands in message's body
@@ -54,7 +54,7 @@ $suffix = '';
 
 $sendmail = '/usr/sbin/sendmail';
 $delivery = 'internal';
-$domain = `/bin/hostname --fqdn`; chomp $domain;
+$domain = `uname -n`; chomp $domain;
 $directory = '/var/spool/minimalist';
 $admin = "postmaster\@$domain";
 $security = 'careful';
@@ -77,6 +77,7 @@ $logmessages = 'no';
 $listinfo = 'yes';
 $strip_rrq = 'no';
 $remove_resent = 'no';
+$modify_msgid = 'no';
 $xtrahdr = '';
 $language = "en";
 $list_gecos = '';
@@ -84,6 +85,8 @@ $background = 'no';
 $to_recipient = 'no';
 # Languages support
 $charset = 'us-ascii';
+$blocked_robots = 'CurrentlyWeAreBlockingNoRobot-Do__NOT__leaveThisBlank';	# -VTV-
+$cc_on_subscribe = 'no';	# -VTV-
 
 ##
 $body_controlled = 0;
@@ -101,8 +104,8 @@ $spaces = '^\s*(.*?)\s*$';
 # 1st form: Vladimir Litovka <doka@kiev.sovam.com>
 # 2nd form: doka@kiev.sovam.com (Vladimir Litovka)
 
-$first = '^(.*?)\s*<(.*?)>.*';		# $1 - gecos, $2 - address
-$second = '^(.*?)\s*\((.*?)\).*';	# $1 - address, $2 - gecos
+$first = '((.*?)\s*<(.*?)>)(.*)';	# $2 - gecos, $3 - address, $4 - rest
+$second = '((.*?)\s*\((.*?)\))(.*)';	# $2 - address, $3 - gecos, $4 - rest
 
 ########################################################
 # >>>>>>>>>>>>>>> SELF - CONFIGURING <<<<<<<<<<<<<<<<< #
@@ -143,17 +146,23 @@ if ($ARGV[0] eq '-') {
 	"Logging: $logfile\n".
 	"Log info about messages: $logmessages\n".
 	"Background execution: $background\n".
-	"Authentication request valid at least $auth_valid hours\n";
+	"Authentication request valid at least $auth_valid hours\n".
+        "Blocked robots:";
+  if ($blocked_robots !~ /__NOT__/) {
+    foreach (split(/\|/, $blocked_robots)) {
+      print "\n\t$_"; }
+   }
+  else { print " no one"; }
 
   if ( @blacklist ) {
-    print "Global access list is:\n";
+    print "\nGlobal access list is:\n";
     foreach (@blacklist) {
       if ( $_ =~ s/^!(.*)//g ) { print "\t - ".$1." allowed\n" }
       else { print "\t - ".$_." disallowed\n" }
      };
    };
 
-  print "\n";
+  print "\n\n";
   while ( $ARGV[0] ) {
     @trusted = (); $xtrahdr = ''; read_config($config, "global");
 
@@ -201,6 +210,8 @@ if ($ARGV[0] eq '-') {
 	"Errors from MTA: ".($errors_to eq 'drop' ? "drop" :
 	          ($errors_to eq 'verp' ? "generate VERP" : "return to $errors_to"))."\n".
 	"Modify subject: $modify_subject\n".
+	"Modify Message-ID: $modify_msgid\n".
+	"Notify on subscribe/unsibscribe event: $cc_on_subscribe\n".
 	"Maximal users per list: ".($maxusers ? $maxusers : "unlimited")."\n".
 	"Maximal recipients per message: ".($maxrcpts ? $maxrcpts : "unlimited")."\n".
 	"Delay between deliveries: ".($delay ? $delay : "none")."\n".
@@ -223,6 +234,8 @@ if ($ARGV[0] eq '-') {
     $msg .= " * Invalid security level '$security'\n" if ($security !~ /^none$|^careful$|^paranoid$/i);
     $msg .= " * Invalid 'copy to sender' value '$copy_sender'\n" if ($copy_sender !~ /^yes$|^no$/i);
     $msg .= " * Invalid 'modify subject' value '$modify_subject'\n" if ($modify_subject !~ /^yes$|^no$|^more$/i);
+    $msg .= " * Invalid 'modify message-id' value '$modify_msgid'\n" if ($modify_msgid !~ /^yes$|^no$/i);
+    $msg .= " * Invalid 'cc on subscribe' value '$cc_on_subscribe'\n" if ($cc_on_subscribe !~ /^yes$|^no$/i);
     $msg .= " * Invalid 'reply-to list' value '$reply_to_list'\n" if ($reply_to_list !~ /^yes$|^no$|\@/i);
     $msg .= " * Invalid 'from' value '$outgoing_from'\n" if ($outgoing_from !~ /\@|^$/i);
     $msg .= " * Invalid authentication request validity time: $auth_valid\n" if ($auth_valid !~ /^[0-9]+$/);
@@ -269,7 +282,7 @@ $from = $reply = $sender = $xsender = $subject = '';
 # with human-like From: header. In most cases "From " is the only way to
 # find out envelope sender of message.
 if ($header =~ /^From (.*)\n/i) {
-  exit 0 if ($1 =~ /MAILER-DAEMON/i); }
+  exit 0 if ($1 =~ /(MAILER-DAEMON|postmaster)@/i); }
 
 # Extract From:
 if ($header =~ /(^|\n)from:\s+(.*\n([ \t]+.*\n)*)/i) {
@@ -289,14 +302,16 @@ if ($header =~ /(^|\n)x-sender: (.*)\n/i) { $xsender = $2; }
 $mailto = ( $reply eq '' ? $from : $reply );
 
 # Preparing From:
-if ($from =~ s/$first/$2/og) { ($gecos = $1) =~ s/$spaces/$1/gs;}
-elsif ($from =~ s/$second/$1/og) { ($gecos = $2) =~ s/$spaces/$1/gs; }
+if ($from =~ s/$first/$3/og) { ($gecos = $2) =~ s/$spaces/$1/gs;}
+elsif ($from =~ s/$second/$2/og) { ($gecos = $3) =~ s/$spaces/$1/gs; }
 $from =~ s/\s+//gs; $from = lc($from);
 
 exit 0 if (($xsender eq $mesender) || ($from eq $mesender));	# LOOP detected
-exit 0 if (($from =~ /MAILER-DAEMON/i) ||
-	   ($sender =~ /MAILER-DAEMON/i) ||
-	   ($xsender =~ /MAILER-DAEMON/i));	# ignore messages from MAILER-DAEMON
+exit 0 if (($from =~ /(MAILER-DAEMON|postmaster)@/i) ||		# -VTV-
+	   ($sender =~ /(MAILER-DAEMON|postmaster)@/i) ||
+	   ($xsender =~ /(MAILER-DAEMON|postmaster)@/i));	# ignore messages from MAILER-DAEMON
+
+exit 0 if ( $header =~ /^($blocked_robots):/);			# disable loops from robots -VTV-
 
 foreach (@blacklist) {				# Parse access control list
   if ( $_ =~ s/^!(.*)//g ) {
@@ -316,7 +331,7 @@ if (!$userpwd && $body =~ s/^\*password:[ \t]+(\w+)\n+//i) {
   $userpwd = $1; }
 
 # Get (multiline) subject
-if ($header =~ /(^|\n)subject:\s+(.*\n([ \t]+.*\n)*)/i) {
+if ($header =~ /(^|\n)subject:[ \t]+(.*\n([ \t]+.*\n)*)/i) {
   $subject = $2; $subject =~ s/$spaces/$1/gs; }
 
 $body =~ s/\n*$/\n/g;
@@ -354,7 +369,7 @@ To: $admin
 Subject: Can not fork
 
 ERROR:
-    Minimalist can't fork due to the following reason:
+    Minimalist can not fork due to the following reason:
 _EOF_
    $forks = 0;
 
@@ -390,9 +405,8 @@ _EOF_
  chdir("$directory/$list");
  read_config("config");
 
- # Check and remove X-BeenThere
- if ($header =~ s/(^|\n)x-beenthere:\s+(.*)\n/$1/i) {
-   exit 0 if ($2 eq "$list\@$domain"); }
+ # Remove or exit per List-ID
+ exit 0 if ($header =~ s/(^|\n)list-id:\s+(.*)\n/$1/i && $2 =~ /$list.$domain/i);
 
  if ($modify_subject ne 'no') {
    $orig_subj = $subject;
@@ -501,18 +515,27 @@ _EOF_
    # used later, when sending message to members except those who already
    # received this message directly.
 
-   if ($header =~ s/(^|\n)to:\s+(.*\n([ \t]+.*\n)*)/$1/i) {
-     $rc = $2; push (@recip, split(/[\n,]+/, $rc)); }
-   if ($header =~ s/(^|\n)cc:\s+(.*\n([ \t]+.*\n)*)/$1/i) {
-     $rc = $2; push (@recip, split(/[\n,]+/, $rc)); }
+   if ($header =~ s/(^|\n)to:\s+(.*\n([ \t]+.*\n)*)/$1/i) { $rc = $2 }
+   if ($header =~ s/(^|\n)cc:\s+(.*\n([ \t]+.*\n)*)/$1/i) { $rc .= ",".$2 }
+
+   if ($rc) {
+     @pre_recip = split(/,/, $rc);
+     foreach $rc (@pre_recip) {
+       if ($rc =~ s/$first/$4/) { push (@recip, $1) }
+       elsif ($rc =~ s/$second/$4/) { push (@recip, $1) }
+       else { $rc =~ s/$spaces/$1/; push (@recip, $rc); }
+      }
+    }
 
    # Search for user's supplied GECOS
    foreach $trcpt (@recip) {
      $trcpt =~ s/$spaces/$1/gs;
+     next if (! $trcpt);  # In case "To: e@mail, \n" - don't push spaces, which are between ',' and '\n'
+
      push(@hdrcpt, $trcpt);
 
-     if ( $trcpt =~ s/$first/$2/g ) { ($tmp_to_gecos = $1) =~ s/$spaces/$1/gs; }
-     elsif ( $trcpt =~ s/$second/$1/g ) { ($tmp_to_gecos = $2) =~ s/$spaces/$1/gs; }
+     if ( $trcpt =~ s/$first/$3/g ) { ($tmp_to_gecos = $2) =~ s/$spaces/$1/gs; }
+     elsif ( $trcpt =~ s/$second/$2/g ) { ($tmp_to_gecos = $3) =~ s/$spaces/$1/gs; }
      push(@rcpts, $trcpt = lc($trcpt));
 
      $to_gecos = $tmp_to_gecos if ($tmp_to_gecos && $trcpt eq $list.'@'.$domain);
@@ -525,7 +548,7 @@ _EOF_
      if ($list_gecos) {
        for ($i=0; $i<@hdrcpt; $i++) {
 	 if ($hdrcpt[$i] =~ /$list\@$domain/) {	# Yes, list's address
-	   $hdrcpt[$i] =~ s/$second/$1/g if (! ($hdrcpt[$i] =~ s/$first/$2/g));
+	   $hdrcpt[$i] =~ s/$second/$2/g if (! ($hdrcpt[$i] =~ s/$first/$3/g));
 	   $hdrcpt[$i] = "$list_gecos <$hdrcpt[$i]>";
 	  }
 	}
@@ -557,9 +580,15 @@ _EOF_
      $header =~ s/x-confirm-reading-to:\s+.*\n//ig;
     }
 
+   if ($modify_msgid eq 'yes') {	# Change Message-ID in outgoing message
+     $header =~ s/message-id:\s+(.*)\n//i;
+     $old_msgid = $1; $old_msgid =~ s/$first/$3/g;
+     $msgid = "MMLID_".int(rand(100000));
+     $header .= "Message-ID: <$msgid-$old_msgid>\n";
+    }
+
    chomp ($header);
-   $header .= "\nPrecedence: list\n";	# For vacation and similar programs
-   $header .= "X-BeenThere: $list\@$domain\n";
+   $header .= "\nPrecedence: list\n";		# For vacation and similar programs
 
    # Remove original Reply-To unconditionally, set configured one if it is
    $header =~ s/(^|\n)reply-to:\s+.*\n([ \t]+.*\n)*/$1/ig;
@@ -591,7 +620,9 @@ _EOF_
      elsif ($listarchive) {
        $header .= "List-Archive: $listarchive\n"; }
     }
+   $header .= "List-ID: <$list.$domain>\n";
    $header .= "X-List-Server: Minimalist v$version <http://www.mml.org.ua/>\n";
+   $header .= "X-BeenThere: $list\@$domain\n";	# This header deprecated due to RFC2919 (List-ID)
    if ($xtrahdr) {
      chomp $xtrahdr; $header .= "$xtrahdr\n"; }
 
@@ -688,7 +719,7 @@ _EOF_
   }
 
  if ($email ne '') {
-   $email =~ s/$first/$2/g || $email =~ s/$second/$1/g ;
+   $email =~ s/$first/$3/g || $email =~ s/$second/$2/g ;
    $email =~ s/\s+//gs; $email = lc($email);
   }
 
@@ -707,6 +738,7 @@ _EOF_
      chdir "$directory/$list";
      read_config("config");
 
+     $msg .= "List-ID: <$list.$domain>\n";
      $owner = "$list-owner\@$domain";
 
      if ($cmd eq 'subscribe' || $cmd eq 'unsubscribe') {
@@ -768,11 +800,9 @@ _EOF_
           cMaxsize => 'maxsize',
           cMode => 'mode');
 
- if (! grep(/^$cmd$/, %cmds)) { # Bad syntax or unknown instruction.
-   $msg =~ s/(^|\n)subject:\s+(.*\n([ \t]+.*\n)*)/$1/i;
-   $msg .= "Subject: ".$msgtxt{'27.0'.$language}."\n\n * $subject *\n".$msgtxt{'27'.$language};
-   $global_exit_status = 10 if ($body_controlled);
-  }
+ $qcmd = quotemeta($cmd);
+ if (! grep(/^$qcmd$/, %cmds)) { # Bad syntax or unknown instruction.
+   goto BadSyntax; }
  elsif ( ($list ne '') && (! -d "$directory/$list") ) {
    $msg .= $msgtxt{'12'.$language}." \U$list\E ".$msgtxt{'13'.$language}.
      " minimalist\@$domain ".$msgtxt{'14'.$language};
@@ -782,12 +812,15 @@ _EOF_
    chdir "$directory/$list";
    read_config("config");
 
+#   exit 0 if ($header =~ /(^|\n)list-id:\s+(.*)\n/i && $2 =~ /$list.$domain/i);
+
+   $msg .= "List-ID: <$list.$domain>\n";
+   $owner = "$list-owner\@$domain";
+
    # Check for possible loop
    $melist = "$list\@$domain";
    exit 0 if (($from eq $melist) || ($email eq $me) || ($email eq $melist));
 
-   $owner = "$list-owner\@$domain";
-
    if (eval($verify)) {
      &logCommand($subject) if (eval("$cmd(1)") && $logfile ne 'none');
     }
@@ -825,20 +858,25 @@ _EOF_
  elsif (($cmd eq $cmds{cWho}) && ($list ne '')) {
    chdir "$directory/$list";
    read_config("config");
+   $msg .= "List-ID: <$list.$domain>\n";
 
    if (eval($verify)) {
      &logCommand($subject) if ($logfile ne 'none');
      $msg .= $msgtxt{'25'.$language}." \U$list\E$suffix:\n\n";
      if (open(LIST, "list".$suffix)) {
        while ($ent = <LIST>) {
-         if ($ent !~ /^#/ && chomp ($ent) && $ent =~ s/(>.*)?$//) {
+         push (@whoers, $ent) if ($ent !~ /^#/ && chomp ($ent)) }
+       if (@whoers) {
+         @whoers = sort @t = Invert ('@', '!', @whoers);
+	 @whoers = Invert ('!', '@', @whoers);
+	 foreach $ent (@whoers) {
+	   $ent =~ s/(>.*)?$//;
 	   $msg .= $ent.&txtUserSet($1);
-	   $whocnt++;
 	  }
 	}
        close LIST;
       }
-     $msg .= $msgtxt{'25.1'.$language}."$whocnt\n";
+     $msg .= $msgtxt{'25.1'.$language}.@whoers."\n";
     }
    else { $msg .= $msgtxt{'26'.$language}; }
   }
@@ -849,6 +887,7 @@ _EOF_
 
    chdir "$directory/$list";
    read_config("config");
+   $msg .= "List-ID: <$list.$domain>\n";
 
    if (eval($verify) || $security ne 'paranoid') {
      &logCommand($subject)
@@ -863,6 +902,7 @@ _EOF_
 
    chdir "$directory/$list";
    read_config("config");
+   $msg .= "List-ID: <$list.$domain>\n";
 
    # Only administrator allowed to change settings
    if (eval($verify)) {
@@ -873,6 +913,12 @@ _EOF_
      $msg .= $msgtxt{'44'.$language};
     }
   }
+ else {
+   BadSyntax:		# LABEL HERE !!!
+     $msg =~ s/(^|\n)subject:\s+(.*\n([ \t]+.*\n)*)/$1/i;
+     $msg .= "Subject: ".$msgtxt{'27.0'.$language}."\n\n * $subject *\n".$msgtxt{'27'.$language};
+     $global_exit_status = 10 if ($body_controlled);
+  }
  }	# Rest commands
 
  cleanAuth();		# Clean old authentication requests
@@ -935,7 +981,8 @@ sub do_MIME_message {
      $header .= "Content-Type: multipart/mixed;\n\tboundary=\"$boundary\"\n";
 
      if ($footer !~ s/^\*hdr:[ \t]+// && $charset) {
-       $footer = "Content-Type: text/plain; charset=$charset; name=\"footer\"\n".
+       $footer = "Content-Type: text/plain; charset=$charset; name=\"footer.txt\"\n".
+		 "Content-Disposition: inline\n".
                  "Content-Transfer-Encoding: 8bit\n\n".$footer;
       }
 
@@ -1062,6 +1109,8 @@ sub subscribe {
   }
  else { open LIST, ">list".$suffix; } 
 
+ $cc .= "$owner," if ( $cc_on_subscribe =~ /yes/i && ! $deny);
+
  if ($cc) {
    chop $cc;
    $msg .= "Cc: $cc\n";
@@ -1090,9 +1139,16 @@ _EOF_
 #................... UNSUBSCRIBE .....................
 sub unsubscribe {
 
- if ($email) { $msg .= "Cc: $email\n" if ($email ne $from); }
+ $cc = "$owner," if ( $cc_on_subscribe =~ /yes/i );
+
+ if ($email) { $cc .= "$email," if ($email ne $from); }
  else { $email = $from; }
 
+ if ($cc) {
+   chop $cc;
+   $msg .= "Cc: $cc\n";
+  }
+
  if (open LIST, "list".$suffix) {
    $users .= $_ while (<LIST>);
    close LIST;
@@ -1236,7 +1292,7 @@ sub chgSettings {
 	}
 
        $members[$i] = $email.$newSet;
-       $currentSet = '>';	# Indicate, that user found
+       $currentSet = '>';	# Indicate, that user found, even if there are no settings
        last;
       }
     }
@@ -1302,6 +1358,10 @@ if (open(CONF, $fname)) {
      $black =~ s/\s+//g;
      @blacklist = expand_lists(split(':', $black));
 
+    } elsif (($_ =~ /^blocked robots/i) && $global) {			# -VTV-
+      # by default no blocking with a string completely replaced here	# -VTV-
+      ($directive, $blocked_robots) = split(/=/, $_, 2);		# -VTV-
+
     } elsif (($_ =~ /^logfile/i) && $global) {
      ($directive, $logfile) = split(/=/, $_, 2);
      $logfile =~ s/$spaces/$1/gs;
@@ -1431,6 +1491,10 @@ if (open(CONF, $fname)) {
      ($directive, $strip_rrq) = split(/=/, $_, 2);
      $strip_rrq =~ s/$spaces/$1/gs; $strip_rrq = lc($strip_rrq);
 
+    } elsif ($_ =~ /^modify message-id/i) {
+     ($directive, $modify_msgid) = split(/=/, $_, 2);
+     $modify_msgid =~ s/$spaces/$1/gs; $modify_msgid = lc($modify_msgid);
+
     } elsif ($_ =~ /^remove resent/i) {
      ($directive, $remove_resent) = split(/=/, $_, 2);
      $remove_resent =~ s/$spaces/$1/gs; $remove_resent = lc($remove_resent);
@@ -1441,9 +1505,13 @@ if (open(CONF, $fname)) {
      $xtrahdr .= "$tmp_xtrahdr\n";
 
     } elsif ($_ =~ /^language/i) {
-      ($directive, $language) = split(/=/, $_, 2);
-      # Case sensitive, don't do lowercase
-      $language =~ s/$spaces/$1/gs;
+     ($directive, $language) = split(/=/, $_, 2);
+     # Case sensitive, don't do lowercase
+     $language =~ s/$spaces/$1/gs;
+
+    } elsif ($_ =~ /^cc on subscribe/i) {			# -VTV-
+     ($directive, $cc_on_subscribe) = split(/=/, $_, 2);	# -VTV-
+     $cc_on_subscribe =~ s/$spaces/$1/gs; $cc_on_subscribe = lc($cc_on_subscribe);
 
     } elsif ($_ =~ /^charset/i) {
       ($directive, $charset) = split(/=/, $_, 2);
@@ -1695,11 +1763,13 @@ sub Invert {
  my $delim = shift (@_);
  my $newdelim = shift (@_);
  my @var = @_;
- my ($i, $us, $dom) = 0;
+ my $restdelim = '>';	# And remember about user's specific flags, which are delimited by '>'
+ my ($i, $us, $dom, $usdom, $rest) = 0;
 
  for (; $i < @var; $i++) {
-   ($us, $dom) = split ($delim, $var[$i]);
-   $var[$i] = $dom.$newdelim.$us;
+   ($usdom, $rest) = split ($restdelim, $var[$i]);
+   ($us, $dom) = split ($delim, $usdom);
+   $var[$i] = $dom.$newdelim.$us.($rest ? $restdelim.$rest : "");
   }
 
  @var;
@@ -1888,4 +1958,184 @@ $msgtxt{'44en'} = "\nERROR:\n\tYou are not allowed to change settings of other p
 # http://www.mml.org.ua/languages/
 #
 
+push (@languages, 'ru');
+
+##################################################################
+###   ru = Russian
+
+#-------------------------------
+$msgtxt{'1ru'} = <<_EOF_ ;
+This is the Minimalist Mailing List Manager.
+
+        (   ), 
+    (   ,     ). 
+       ,    
+,    'body' ( ).  
+     'stop'  'exit' ( ),  
+ 10  .
+
+  :
+
+subscribe <list> [<email>] :
+        <list>.   <list> 
+     '-writers',      <list>,  
+      ,    .
+
+unsubscribe <list> [<email>] :
+       <list>.     
+    '-writers' (   . )
+
+auth <code> :
+      .    
+       ,        
+    .
+
+mode <list> <email> <mode> :
+           
+     .     
+    .   'mode'    
+    (  ):
+      * 'reader' -       ;
+      * 'writer' -       
+                        ;
+      * 'usual' -       . 
+		      
+		  .
+      * 'suspend' -    
+      * 'resume' -    
+      * 'maxsize <size>' -     (
+			   ),   
+			      .
+      * 'reset' -    .
+    
+suspend <list>:
+        
+
+resume <list>:
+        'suspend'  
+     
+
+maxsize <list> <size>:
+        ( ),  
+        .
+
+which [<email>] :
+       ,    
+
+info [<list>] :
+           
+      <list>
+
+who <list> :
+         <list>
+
+help :
+       ( )
+
+ ,  <email>      'who'  'mode'
+     (, 
+,   auth   
+).      .  
+ ,     ,   :
+
+{pwd: list_password}
+
+:
+
+To: MML Discussion {pwd: password1235} <mml-general\@kiev.sovam.com>
+
+       ,   
+   .
+_EOF_
+
+#-------------------------------
+$msgtxt{'2ru'} = ":\n\t";
+$msgtxt{'3ru'} = "    .\n\n".
+		 " :\n\t   ";
+$msgtxt{'4ru'} = " \n\t'help' ( )     ,  .\n\n".
+		 "   :";
+#-------------------------------
+$msgtxt{'5ru'} = ":\n\t       .\n\n".
+		 "   :";
+#-------------------------------
+$msgtxt{'6ru'} = ":\n\t      (";
+$msgtxt{'7ru'} = ").\n\n :\n\t   ,   ,\n\t    .\n\n".
+		 "===========================================================================\n".
+		 "   :";
+#-------------------------------
+$msgtxt{'8ru'} = "\n:\n\t        : ";
+$msgtxt{'9ru'} = "\n\n :\n\t,     .\n";
+
+#-------------------------------
+$msgtxt{'10ru'} = "\n:\n\t     ,     .\n".
+		  "\n :\n\t,      :)";
+#-------------------------------
+$msgtxt{'11ru'} = "\n      ";
+#-------------------------------
+$msgtxt{'12ru'} = "\n:\n\t    ";
+$msgtxt{'13ru'} = ".\n\n :\n\t   ";
+$msgtxt{'14ru'} = " \n\t'info' ( )      .\n";
+#-------------------------------
+$msgtxt{'15ru'} = "\n:\n\t         .\n".
+		  "\n :\n\t,      :)";
+#-------------------------------
+$msgtxt{'16ru'} = "\n:\n\t,         .\n".
+		  "\n :\n\t ,   ?      ";
+#-------------------------------
+$msgtxt{'17ru'} = "\n:\n\t,        .\n".
+		  "\n :\n\t ,   ?      ";
+#-------------------------------
+$msgtxt{'18ru'} = " ";
+$msgtxt{'19ru'} = "  .        ";
+$msgtxt{'20ru'} = "(    'Reply'    )\n  :";
+$msgtxt{'21ru'} = "     ";
+$msgtxt{'22ru'} = "     .     \n .\n";
+#-------------------------------
+$msgtxt{'23ru'} = "\n     ";
+#-------------------------------
+$msgtxt{'24ru'} = "\n      ";
+#-------------------------------
+$msgtxt{'25ru'} = "\n ,  ";
+$msgtxt{'25.1ru'} = "\n : ";
+#-------------------------------
+$msgtxt{'26ru'} = "\n:\n\t     ,    .";
+#-------------------------------
+$msgtxt{'27.0ru'} = "     ";
+$msgtxt{'27ru'} = "\n:\n\t".$msgtxt{'27.0ru'}.".\n\n :\n\n".$msgtxt{'1ru'};
+#-------------------------------
+$msgtxt{'28ru'} = "Sincerely, the Minimalist";
+#-------------------------------
+$msgtxt{'29ru'} = "     ";
+#-------------------------------
+$msgtxt{'30ru'} = " ,       (";
+#-------------------------------
+$msgtxt{'31ru'} = "    ";
+$msgtxt{'32ru'} = ".\n\n,    :\n";
+#-------------------------------
+$msgtxt{'33ru'} = "     ";
+$msgtxt{'34ru'} = "  ";
+$msgtxt{'35ru'} = "      , ,   \n ";
+#-------------------------------
+$msgtxt{'36ru'} = "\n ";
+$msgtxt{'37ru'} = "   .\n";
+#-------------------------------
+$msgtxt{'38ru'} = "\n   ;   .".
+		  "\n,  ,    ";
+$msgtxt{'38.1ru'} = "      ";
+#-------------------------------
+$msgtxt{'39ru'} = "       .\n";
+#-------------------------------
+$msgtxt{'40ru'} = "\n(-)";
+#-------------------------------
+$msgtxt{'41ru'} = "\n    ";
+$msgtxt{'42ru'} = "   ";
+$msgtxt{'43ru'} = "  ";
+$msgtxt{'43.1ru'} = "  ";
+$msgtxt{'43.2ru'} = "   ";
+$msgtxt{'43.3ru'} = "  ";
+$msgtxt{'43.4ru'} = "    (): ";
+#-------------------------------
+$msgtxt{'44ru'} = "\n:\n\t         .\n".
+		  "\n:\n\t.";
+
 }
